// set the dimensions and margins of the graph
const bc_margin = {
    top: 50,
    right: 400,
    bottom: 45,
    left: 75,
  },
  bc_width = 945 - bc_margin.left - bc_margin.right,
  bc_height = 472 - bc_margin.top - bc_margin.bottom;

// append the svg object to the body of the page
const bc_svg = d3
  .select('#bubble_chart')
  .append('svg')
  .attr(
    'width',
    bc_width + bc_margin.left + bc_margin.right,
  )
  .attr(
    'height',
    bc_height + bc_margin.top + bc_margin.bottom,
  )
  .append('g')
  .attr(
    'transform',
    `translate(${bc_margin.left},${bc_margin.top})`,
  );

// read the data
d3.csv('iris.csv').then((data) => {
  // add chart title
  bc_svg
    .append('text')
    .attr('x', bc_width / 2)
    .attr('y', -20)
    .attr('text-anchor', 'middle')
    .style('font-size', '22px')
    .text('Sepal Length vs. Sepal Width by Species');

  // add x axis with descriptive label
  const x = d3
    .scaleLinear()
    .domain([3, 8])
    .range([0, bc_width]);
  bc_svg
    .append('g')
    .attr('transform', `translate(0, ${bc_height})`)
    .call(d3.axisBottom(x).ticks(8).tickSizeOuter(0))
    .append('text')
    .attr('x', bc_width / 2)
    .attr('y', 40)
    .attr('text-anchor', 'middle')
    .text('Sepal Length (cm)'); // Update x-axis label

  // add y axis with descriptive label
  const y = d3
    .scaleLinear()
    .domain([1.5, 5])
    .range([bc_height, 0]);
  bc_svg
    .append('g')
    .call(d3.axisLeft(y).ticks(6).tickSizeOuter(0))
    .append('text')
    .attr('text-anchor', 'middle')
    .attr('x', -bc_height / 2)
    .attr('y', -50)
    .text('Sepal Width (cm)') // Update y-axis label
    .attr('transform', 'rotate(-90)');

  // add x axis label
  bc_svg
    .append('text')
    .attr('x', bc_width / 2)
    .attr('y', bc_height + 40) // Position below the x-axis
    .attr('text-anchor', 'middle')
    .text('Sepal Length (cm)');

  // add y axis label
  bc_svg
    .append('text')
    .attr('text-anchor', 'middle')
    .attr('x', -bc_height / 2)
    .attr('y', -60) // Move it a bit further away from the y-axis
    .text('Sepal Width (cm)')
    .attr('transform', 'rotate(-90)');

  // add a scale for bubble size
  const z = d3
    .scaleLinear()
    .domain([0, d3.max(data, (d) => +d.petal_length)])
    .range([1, 40]);

  // add a scale for bubble color
  const species = Array.from(
    new Set(data.map((d) => d.species)),
  );
  const speciesColors = d3
    .scaleOrdinal()
    .domain(species)
    .range(d3.schemeCategory10);

  // create a tooltip div that is hidden by default:
  const tooltip = d3
    .select('#bubble_chart')
    .append('div')
    .style('opacity', 0)
    .attr('class', 'tooltip')
    .style('background-color', 'black')
    .style('border-radius', '5px')
    .style('padding', '10px')
    .style('color', 'white')
    .style('font-family', 'sans-serif');

  // functions to show / update / hide the tooltip
  const showTooltip = (event, d) => {
    tooltip.transition().duration(200);
    tooltip
      .style('opacity', 1)
      .html(
        `Species: ${d.species}<br/>Sepal Length: ${d.sepal_length}<br/>Sepal Width: ${d.sepal_width}`,
      )
      .style('left', event.x + 25 + 'px')
      .style('top', event.y + 25 + 'px');
  };

  const moveTooltip = (event, d) => {
    tooltip
      .style('left', event.x + 25 + 'px')
      .style('top', event.y + 25 + 'px');
  };

  const hideTooltip = (event, d) => {
    tooltip.transition().duration(200).style('opacity', 0);
  };

  // add dots
  bc_svg
    .selectAll('dot')
    .data(data)
    .join('circle')
    .attr('class', 'bubbles')
    .attr('cx', (d) => x(d.sepal_length))
    .attr('cy', (d) => y(d.sepal_width))
    .attr('r', (d) => z(d.petal_length))
    .style('fill', (d) => speciesColors(d.species))
    .attr('stroke', 'white')
    .attr('stroke-width', 1)
    .style('opacity', 0.5)
    .on('mouseover', showTooltip)
    .on('mousemove', moveTooltip)
    .on('mouseleave', hideTooltip);

  // add legend: circles
  const legend = bc_svg
    .selectAll('.legend')
    .data(species)
    .enter()
    .append('g')
    .attr('class', 'legend')
    .attr('transform', (d, i) => `translate(0,${i * 20})`);

  legend
    .append('circle')
    .attr('cx', bc_width + 100)
    .attr('cy', 0)
    .attr('r', 6)
    .style('fill', (d) => speciesColors(d));

  legend
    .append('text')
    .attr('x', bc_width + 120)
    .attr('y', 5)
    .text((d) => d)
    .style('font-size', '14px')
    .attr('alignment-baseline', 'middle');
});
